@@ -147,6 +147,7 @@ module Text.Pandoc.Builder ( module Text.Pandoc.Definition
                            -- * Block list builders
                            , para
                            , plain
+                           , lineBlock
                            , codeBlockWith
                            , codeBlock
                            , rawBlock
@@ -166,16 +167,14 @@ where
 import Text.Pandoc.Definition
 import Data.String
 import Data.Monoid
-import Data.Maybe (fromMaybe)
 import qualified Data.Map as M
 import Data.Sequence (Seq, (|>), viewr, viewl, ViewR(..), ViewL(..))
 import qualified Data.Sequence as Seq
+import Data.Traversable (Traversable)
 import Data.Foldable (Foldable)
 import qualified Data.Foldable as F
-import Data.List (groupBy, intersperse)
+import Data.List (groupBy)
 import Data.Data
-import Data.Typeable
-import Data.Traversable
 import Control.Arrow ((***))
 import GHC.Generics (Generic)
 
@@ -430,6 +429,9 @@ plain ils = if isNull ils
                then mempty
                else singleton . Plain . toList $ ils
 
+lineBlock :: [Inlines] -> Blocks
+lineBlock = singleton . LineBlock . map toList
+
 -- | A code block with attributes.
 codeBlockWith :: Attr -> String -> Blocks
 codeBlockWith attrs = singleton . CodeBlock attrs
